/* Firebird serial register offsets */
#define FBD_SERIAL_OFFSET_TX_DATA                     0x000       /* Write only */
#define FBD_SERIAL_OFFSET_RX_DATA                     0x004       /* Read only */
#define FBD_SERIAL_OFFSET_INT_CTRL                    0x008       /* Write only - soft copy required */
#define FBD_SERIAL_OFFSET_INT_STAT                    0x00C       /* Read only */
#define FBD_SERIAL_OFFSET_CTRL                        0x010       /* Write only - soft copy required */
#define FBD_SERIAL_OFFSET_STAT                        0x014       /* Read only */
#define FBD_SERIAL_OFFSET_BAUD                        0x018       /* Write only - soft copy used */
#define FBD_SERIAL_OFFSET_RS485_CTRL                  0x01C       /* Write only - soft copy required */

#define FBD_SERIAL_BITS_RS485_CTRL_AUTO               0x00010000  /* 1 = Auto (default = 1) */
#define FBD_SERIAL_BITS_RS485_CTRL_TX_ENABLE          0x00020000  /* 1 = Enabled (default = 0) */
#define FBD_SERIAL_BITS_RS485_CTRL_DELAY_ON_ENABLE    0x00040000  /* 1 = Enable control of delay, 0 = delay = 1 baud clock or 16 oversample clocks*/
#define FBD_SERIAL_BITS_RS485_CTRL_DELAY_OFF_MASK     0x000000FF
#define FBD_SERIAL_BITS_RS485_CTRL_DELAY_OFF_DEFAULT  1           /* In 16x oversample clocks */
#define FBD_SERIAL_BITS_RS485_CTRL_DELAY_OFF_MAX      255
#define FBD_SERIAL_BITS_RS485_CTRL_DELAY_OFF_SHIFT    0
#define FBD_SERIAL_BITS_RS485_CTRL_DELAY_ON_MASK      0x0000FF00
#define FBD_SERIAL_BITS_RS485_CTRL_DELAY_ON_DEFAULT   16          /* In 16x oversample clocks */
#define FBD_SERIAL_BITS_RS485_CTRL_DELAY_ON_MAX       255
#define FBD_SERIAL_BITS_RS485_CTRL_DELAY_ON_SHIFT     8

/* FBD_SERIAL_OFFSET_TX_DATA definitions */
#define FBD_SERIAL_BITS_TX_DATA_MASK                  0x000000FF

/* FBD_SERIAL_OFFSET_RX_DATA definitions */
#define FBD_SERIAL_BITS_RX_DATA_MASK                  0x000000FF
#define FBD_SERIAL_BITS_RX_ERROR_BREAK                0x00000100
#define FBD_SERIAL_BITS_RX_ERROR_FRAME                0x00000200
#define FBD_SERIAL_BITS_RX_ERROR_PARITY               0x00000400

/* FBD_SERIAL_OFFSET_INT_CTRL definitions */
#define FBD_SERIAL_BITS_INT_CTRL_TXEMPTY_ENABLE       0x00000001
#define FBD_SERIAL_BITS_INT_CTRL_RXDATA_ENABLE        0x00000002
#define FBD_SERIAL_BITS_INT_CTRL_TXHALF_ENABLE        0x00000004
#define FBD_SERIAL_BITS_INT_CTRL_TSREMPTY_ENABLE      0x00000008
#define FBD_SERIAL_BITS_INT_CTRL_GLOBAL_ENABLE        0x00008000
#define FBD_SERIAL_BITS_INT_CTRL_TXEMPTY_CLEAR        0x00010000
#define FBD_SERIAL_BITS_INT_CTRL_RXDATA_CLEAR         0x00020000
#define FBD_SERIAL_BITS_INT_CTRL_TXHALF_CLEAR         0x00040000

/* FBD_SERIAL_OFFSET_INT_STAT definitions */
#define FBD_SERIAL_BITS_INT_STAT_TXEMPTY_ASSERTED     0x00000001
#define FBD_SERIAL_BITS_INT_STAT_RXDATA_ASSERTED      0x00000002
#define FBD_SERIAL_BITS_INT_STAT_TXHALF_ASSERTED      0x00000004
#define FBD_SERIAL_BITS_INT_CTRL_TSREMPTY_ASSERTED    0x00000008
#define FBD_SERIAL_BITS_INT_STAT_TXEMPTY_PENDING      0x00010000
#define FBD_SERIAL_BITS_INT_STAT_RXDATA_PENDING       0x00020000
#define FBD_SERIAL_BITS_INT_STAT_TXHALF_PENDING       0x00040000

/* FBD_SERIAL_OFFSET_CTRL definitions */
#define FBD_SERIAL_BITS_CTRL_DATA_BITS_5              0x00000000
#define FBD_SERIAL_BITS_CTRL_DATA_BITS_6              0x00000001
#define FBD_SERIAL_BITS_CTRL_DATA_BITS_7              0x00000002
#define FBD_SERIAL_BITS_CTRL_DATA_BITS_8              0x00000003

#define FBD_SERIAL_BITS_CTRL_PARITY_NONE              0x00000000
#define FBD_SERIAL_BITS_CTRL_PARITY_ODD               0x00000010
#define FBD_SERIAL_BITS_CTRL_PARITY_EVEN              0x00000014
#define FBD_SERIAL_BITS_CTRL_PARITY_MARK              0x00000018
#define FBD_SERIAL_BITS_CTRL_PARITY_SPACE             0x0000001C

#define FBD_SERIAL_BITS_CTRL_STOP_1                   0x00000000
#define FBD_SERIAL_BITS_CTRL_STOP_1_5                 0x00000020
#define FBD_SERIAL_BITS_CTRL_STOP_2                   0x00000040

#define FBD_SERIAL_BITS_CTRL_BREAK                    0x00000080
#define FBD_SERIAL_BITS_CTRL_TX_OVERRUN_CLEAR         0x00000100
#define FBD_SERIAL_BITS_CTRL_RX_OVERRUN_CLEAR         0x00000200

#define FBD_SERIAL_BITS_CTRL_TX_FIFO_RESET            0x20000000
#define FBD_SERIAL_BITS_CTRL_RX_FIFO_RESET            0x40000000
#define FBD_SERIAL_BITS_CTRL_GLOBAL_RESET             0x80000000

/* FBD_SERIAL_OFFSET_STAT definitions */
#define FBD_SERIAL_BITS_STAT_TX_FIFO_MASK             0x000000FF
#define FBD_SERIAL_BITS_STAT_TX_FIFO_OVERRUN          0x00008000
#define FBD_SERIAL_BITS_STAT_RX_FIFO_MASK             0x00FF0000
#define FBD_SERIAL_BITS_STAT_RX_FIFO_SHIFT            16
#define FBD_SERIAL_BITS_STAT_RX_FIFO_OVERRUN          0x80000000

#define FBD_SERIAL_RX_FIFO_MAX_DEPTH                  64
#define FBD_SERIAL_TX_FIFO_MAX_DEPTH                  64

#define FBD_SERIAL_BAUD_CLOCK                         250000000
#define FBD_SERIAL_PRESCALE                           16
