#include "precomp.h"

static unsigned long aslcxp_timeout_jiffies(unsigned int ms)
{
   unsigned long jiffies_now = jiffies;
   unsigned long jiffies_to_expire = jiffies_now + msecs_to_jiffies(ms);
   if (jiffies_to_expire <= (jiffies_now + 1)) jiffies_to_expire = jiffies_now + 1;
   return jiffies_to_expire;
}

#if LINUX_VERSION_CODE < KERNEL_VERSION(4,15,0)

static void old_ack_timer_callback(unsigned long data)
{
   struct cxp_ack_timer *ack_timer = (struct cxp_ack_timer *)data;
   struct cxp_transaction * transaction = container_of(ack_timer, struct cxp_transaction, ack_timer);
   ack_timer->callback(transaction); 
   return;
}

void aslcxp_init_ack_timer(struct cxp_ack_timer *ack_timer, void (*callback)(struct cxp_transaction *))
{
   ack_timer->callback = callback;
   setup_timer(&ack_timer->ack_timer_list, old_ack_timer_callback, (unsigned long)ack_timer);
   return;
}

void aslcxp_start_ack_timer(struct cxp_ack_timer *ack_timer, unsigned int ms)
{
   ack_timer->ack_timer_list.expires = aslcxp_timeout_jiffies(ms);
   add_timer(&ack_timer->ack_timer_list);
   return;
}

void aslcxp_restart_ack_timer(struct cxp_ack_timer *ack_timer, unsigned int ms)
{
   mod_timer(&ack_timer->ack_timer_list, aslcxp_timeout_jiffies(ms));
   return;
}

void aslcxp_stop_ack_timer(struct cxp_ack_timer *ack_timer)
{
   del_timer_sync(&ack_timer->ack_timer_list);
   return;
}

void aslcxp_delete_ack_timer(struct cxp_ack_timer *ack_timer, unsigned int ms)
{
   return;
}

#else

static void ack_timer_callback(struct timer_list *expired_ack_timer_list)
{
   struct cxp_ack_timer * ack_timer = container_of(expired_ack_timer_list, struct cxp_ack_timer, ack_timer_list);
   struct cxp_transaction * transaction = container_of(ack_timer, struct cxp_transaction, ack_timer);
   ack_timer->callback(transaction); 
}

void aslcxp_init_ack_timer(struct cxp_ack_timer *ack_timer, void (*callback)(struct cxp_transaction *))
{
   ack_timer->callback = callback;
   timer_setup(&ack_timer->ack_timer_list, ack_timer_callback, 0);
   return;
}

void aslcxp_start_ack_timer(struct cxp_ack_timer *ack_timer, unsigned int ms)
{
   ack_timer->ack_timer_list.expires = aslcxp_timeout_jiffies(ms);
   add_timer(&ack_timer->ack_timer_list);
   return;
}

void aslcxp_restart_ack_timer(struct cxp_ack_timer *ack_timer, unsigned int ms)
{
   mod_timer(&ack_timer->ack_timer_list, aslcxp_timeout_jiffies(ms));
   return;
}

void aslcxp_stop_ack_timer(struct cxp_ack_timer *ack_timer)
{
   del_timer_sync(&ack_timer->ack_timer_list);
   return;
}

void aslcxp_delete_ack_timer(struct cxp_ack_timer *ack_timer, unsigned int ms)
{
   return;
}

#endif
