#include "precomp.h"

extern int aslcxp_hw_debug;

int aslcxp_decode_hw_response(uint32_t response, uint8_t *decoded)
{
   int success = 0;
   uint32_t dwMatchCount = 0;
   uint8_t *pIn = (uint8_t*)&response;

   if (pIn[0] == pIn[1]) dwMatchCount++;
   if (pIn[0] == pIn[2]) dwMatchCount++;
   if (pIn[0] == pIn[3]) dwMatchCount++;
   if (dwMatchCount >= 2)
   {
      *decoded = pIn[0];
      success = 1;
      goto bail;
   }

   dwMatchCount = 0;
   if (pIn[1] == pIn[2]) dwMatchCount++;
   if (pIn[1] == pIn[3]) dwMatchCount++;
   if (dwMatchCount >= 2)
   {
      *decoded = pIn[1];
      success = 1;
      goto bail;
   }

bail:
   return success;
}

void aslcxp_hw_read_fifo(struct aslcxp_device *cxp_device, void *pDestination, uint32_t dwCountBytesToHost)
{
   uint8_t *pHostBuffer = (uint8_t *)pDestination;

   if (aslcxp_hw_debug) dev_printk(KERN_INFO, cxp_device->device, "%s: %d bytes\n", __FUNCTION__, dwCountBytesToHost);

   if (dwCountBytesToHost > 255) dwCountBytesToHost = 0;

   while (dwCountBytesToHost)
   {
      uint32_t dwFifoData = aslcxp_read32(cxp_device, FBD_CXP_ADDR_TO_HOST_FIFO);
      if (pHostBuffer != NULL){
         *pHostBuffer++ = (uint8_t)(dwFifoData >> 24);
         if (dwCountBytesToHost > 1) *pHostBuffer++ = (uint8_t)(dwFifoData >> 16);
         if (dwCountBytesToHost > 2) *pHostBuffer++ = (uint8_t)(dwFifoData >> 8);
         if (dwCountBytesToHost > 3) *pHostBuffer++ = (uint8_t)dwFifoData;
      }
      if (dwCountBytesToHost > 4) dwCountBytesToHost -= 4;
      else dwCountBytesToHost = 0;
   }
   return;
}

int aslcxp_hw_is_transaction_outstanding_locked(struct cxp_transaction *transaction)
      {
   int transaction_outstanding = 0;

   switch(transaction->cxp_transaction_state){
   case cxp_started:
   case cxp_tentative_ack:
   case cxp_wait_ack:
      transaction_outstanding = 1;
         break;
      default:
         break;
      }
   return transaction_outstanding;
   }

uint32_t aslcxp_hw_count_bytes_to_host_fifo(struct aslcxp_device *cxp_device)
{
   return aslcxp_read32(cxp_device, FBD_CXP_ADDR_TO_HOST_BYTE_COUNT);
}

int aslcxp_hw_process_read_ack_locked(struct aslcxp_device *cxp_device)
   {
   struct cxp_transaction * transaction = &cxp_device->cxp_transaction;
   uint32_t dwFifoBytesToHost = 0;
   uint32_t dwBytesRequested = 0;
   uint32_t dwBytesToTransfer = 0;
   uint32_t dwBytesRemaining = 0;
   uint32_t protocol_error_code = 0;
   long     error_code = 0;
   int      wake_sleeper = 0;

   dwFifoBytesToHost = aslcxp_hw_count_bytes_to_host_fifo(cxp_device);

   if (aslcxp_hw_debug) dev_printk(KERN_INFO, cxp_device->device, "%s: %u bytes in fifo\n", __FUNCTION__, dwFifoBytesToHost);

   if ((!aslcxp_hw_is_transaction_outstanding_locked(transaction)) || (transaction->ioctl != (unsigned int)MDACXP_IOCTL_CXP_READ)){
      if (aslcxp_hw_debug) dev_printk(KERN_INFO, cxp_device->device, "%s Dumping receive FIFO\n", __FUNCTION__);
      aslcxp_hw_read_fifo(cxp_device, NULL, dwFifoBytesToHost);
      goto bail;
   }

   dwBytesRequested = transaction->cxpheader.dwByteCount;
   if (dwFifoBytesToHost == dwBytesRequested){
      dwBytesToTransfer = dwBytesRequested;
      dwBytesRemaining = 0;
      protocol_error_code = 0;
  }
  else if (dwFifoBytesToHost > dwBytesRequested){
      dwBytesToTransfer = dwBytesRequested;
      dwBytesRemaining = dwFifoBytesToHost - (4 * ((dwBytesToTransfer + 3) / 4));
      error_code = -EIO;
      protocol_error_code = FBD_CXP_ASL_VALUE_TO_HOST_ERROR_SIZE_MISMATCH;
   }
   else{
      dwBytesToTransfer = dwFifoBytesToHost;
      error_code = -EIO;
      protocol_error_code = FBD_CXP_ASL_VALUE_TO_HOST_ERROR_SIZE_MISMATCH;
   }

   if (aslcxp_hw_debug) dev_printk(KERN_INFO, cxp_device->device, "%s: %u of %u bytes available, %u extra\n", __FUNCTION__, dwFifoBytesToHost, dwBytesRequested, dwBytesRemaining);

   aslcxp_hw_read_fifo(cxp_device, &transaction->cxpdata->baData[0], dwBytesToTransfer);
   transaction->cxpheader.dwByteCount = dwBytesToTransfer;
   if (dwBytesRemaining) aslcxp_hw_read_fifo(cxp_device, NULL, dwBytesRemaining);

   wake_sleeper = aslcxp_complete_transaction_locked(transaction, error_code, protocol_error_code);
   
bail:
   return wake_sleeper;
}

int aslcxp_hw_process_write_ack_locked(struct aslcxp_device *cxp_device)
      {
   int wake_sleeper = 0;
   struct cxp_transaction * transaction = &cxp_device->cxp_transaction;

   if (aslcxp_hw_debug) dev_printk(KERN_INFO, cxp_device->device, "%s\n", __FUNCTION__);
   if (aslcxp_hw_debug) dev_printk(KERN_INFO, cxp_device->device, "%s: %d bytes in FIFO\n", __FUNCTION__, aslcxp_hw_count_bytes_to_host_fifo(cxp_device));
   if (!aslcxp_hw_is_transaction_outstanding_locked(transaction)) goto bail;
   if (aslcxp_hw_debug) dev_printk(KERN_INFO, cxp_device->device, "%s transaction outstanding\n", __FUNCTION__);
   if (transaction->ioctl != (unsigned int)MDACXP_IOCTL_CXP_WRITE) goto bail;
   if (aslcxp_hw_debug) dev_printk(KERN_INFO, cxp_device->device, "%s transaction matches\n", __FUNCTION__);

   transaction->error_code = 0;
   transaction->protocol_error_code = 0;
   transaction->cxp_transaction_state = cxp_complete;
   wake_sleeper = 1;

bail:
   return wake_sleeper;
}

int aslcxp_hw_process_reset_ack_locked(struct aslcxp_device *cxp_device)
            {
   int wake_sleeper = 0;
   struct cxp_transaction * transaction = &cxp_device->cxp_transaction;

   if (aslcxp_hw_debug) dev_printk(KERN_INFO, cxp_device->device, "%s\n", __FUNCTION__);

   if (!aslcxp_hw_is_transaction_outstanding_locked(transaction)) goto bail;
   if (transaction->ioctl != (unsigned int)MDACXP_IOCTL_RESET_DEVICE) goto bail;

   transaction->error_code = 0;
   transaction->protocol_error_code = 0;
   transaction->cxp_transaction_state = cxp_complete;
   wake_sleeper = 1;

bail:
   return wake_sleeper;
            }

int aslcxp_hw_process_tentative_ack_locked(struct aslcxp_device *cxp_device)
{
   int wake_sleeper = 0;
   struct cxp_transaction * transaction = &cxp_device->cxp_transaction;

   if (aslcxp_hw_debug) dev_printk(KERN_INFO, cxp_device->device, "%s\n", __FUNCTION__);

   switch(transaction->cxp_transaction_state){
      case cxp_started:
      case cxp_tentative_ack:
         transaction->cxp_transaction_state = cxp_tentative_ack;
         transaction->ack_timeout_ms = 200;
         wake_sleeper = 1;
            break;
         default:
            break;
      }

   return wake_sleeper;
}

uint32_t aslcxp_swab(uint32_t arg)
{
   uint32_t retval = 0;
   uint8_t *pByteIn = (uint8_t *)&arg;
   uint8_t *pByteOut = (uint8_t *)&retval;

   pByteOut[0] = pByteIn[3];
   pByteOut[1] = pByteIn[2];
   pByteOut[2] = pByteIn[1];
   pByteOut[3] = pByteIn[0];

   return retval;
}

int aslcxp_hw_process_wait_ack_locked(struct aslcxp_device *cxp_device)
{
   int wake_sleeper = 0;
   uint32_t dwFifoBytesToHost = 0;
   struct cxp_transaction * transaction = &cxp_device->cxp_transaction;

   if (aslcxp_hw_debug) dev_printk(KERN_INFO, cxp_device->device, "%s\n", __FUNCTION__);

   if (transaction->cxp_transaction_state != cxp_started) goto bail;

   dwFifoBytesToHost = aslcxp_hw_count_bytes_to_host_fifo(cxp_device);
   if (dwFifoBytesToHost >= sizeof(transaction->ack_timeout_ms)){
      aslcxp_hw_read_fifo(cxp_device, &transaction->ack_timeout_ms, sizeof(transaction->ack_timeout_ms));
      transaction->ack_timeout_ms = aslcxp_swab(transaction->ack_timeout_ms);
      if (transaction->ack_timeout_ms){
         transaction->cxp_transaction_state = cxp_wait_ack;
         wake_sleeper = 1;
      }
   }

   if (aslcxp_hw_debug) dev_printk(KERN_INFO, cxp_device->device, "%s: %u ms\n", __FUNCTION__, transaction->ack_timeout_ms);

bail:
   return wake_sleeper;
}

int aslcxp_hw_process_device_reponse_locked(struct aslcxp_device *cxp_device)
{
   int wake_sleeper = 0;
   uint32_t dwAcknowledgeCode = 0;
   uint8_t  bAcknowledgeCode = 0;
   int fSuccess = 1;

   dwAcknowledgeCode = aslcxp_read32(cxp_device, FBD_CXP_ADDR_TO_HOST_ACK);
   if (aslcxp_hw_debug) dev_printk(KERN_INFO, cxp_device->device, "%s: %08X\n", __FUNCTION__, dwAcknowledgeCode);
   fSuccess = aslcxp_decode_hw_response(dwAcknowledgeCode, &bAcknowledgeCode);
   if (aslcxp_hw_debug) dev_printk(KERN_INFO, cxp_device->device, "%s: %02X, %s\n", __FUNCTION__, (unsigned int)bAcknowledgeCode, fSuccess ? "OK" : "Invalid");
   if (!fSuccess) goto bail;

   switch(bAcknowledgeCode){
   case FBD_CXP_VALUE_TO_HOST_ACK_READ_OK:
      wake_sleeper = aslcxp_hw_process_read_ack_locked(cxp_device);
      break;
   case FBD_CXP_VALUE_TO_HOST_ACK_WRITE_OK:
      wake_sleeper = aslcxp_hw_process_write_ack_locked(cxp_device);
      break;
   case FBD_CXP_VALUE_TO_HOST_RST_OK:
      wake_sleeper = aslcxp_hw_process_reset_ack_locked(cxp_device);
      break;
   case FBD_CXP_VALUE_TO_HOST_TENTATIVE_ACK:
      wake_sleeper = aslcxp_hw_process_tentative_ack_locked(cxp_device);
      break;
   case FBD_CXP_VALUE_TO_HOST_WAIT_ACK:
      wake_sleeper = aslcxp_hw_process_wait_ack_locked(cxp_device);
      break;
   case FBD_CXP_VALUE_TO_HOST_ERROR_INVALID_ADDRESS:
   case FBD_CXP_VALUE_TO_HOST_ERROR_INVALID_DATA:
   case FBD_CXP_VALUE_TO_HOST_ERROR_INVALID_CONTOL:
   case FBD_CXP_VALUE_TO_HOST_ERROR_INVALID_WRITE:
   case FBD_CXP_VALUE_TO_HOST_ERROR_INVALID_READ:
   case FBD_CXP_VALUE_TO_HOST_ERROR_SIZE_TOO_LARGE:
   case FBD_CXP_VALUE_TO_HOST_ERROR_INVALID_SIZE:
   case FBD_CXP_VALUE_TO_HOST_ERROR_INCORERCT_CRC:
   default:
      wake_sleeper = aslcxp_complete_transaction_locked(&cxp_device->cxp_transaction, -EIO, (uint32_t)bAcknowledgeCode);
      break;
   }

bail:
   return wake_sleeper;
}

irqreturn_t aslcxp_isr(struct mda_device *mda_device, void *data)
{
   irqreturn_t irqreturn = IRQ_NONE;
   struct aslcxp_device *cxp_device = data;
   int wake_sleeper = 0;
   uint32_t dwInterruptStatus = 0;
   uint32_t dwInterruptFifoLevel = 0;

   spin_lock(&cxp_device->interrupt_spinlock);

   dwInterruptStatus = aslcxp_read32(cxp_device, FBD_CXP_ADDR_IRQ_STATUS);
   if (aslcxp_hw_debug) dev_printk(KERN_INFO, cxp_device->device, "%s: Interrupt status = 0x%08X\n", __FUNCTION__, dwInterruptStatus);

   if (dwInterruptStatus & FBD_CXP_VALUE_IRQ_STATUS_EVENT_QUEUE_OVERFLOW)
   {
      if (aslcxp_hw_debug) dev_printk(KERN_ERR, cxp_device->device, "%s: Interrupt FIFO Overflow\n", __FUNCTION__);
      irqreturn = IRQ_HANDLED;
      aslcxp_hw_panic_locked(cxp_device); /* Emergency stop! */
      goto bail;
   }

   if (dwInterruptStatus & FBD_CXP_VALUE_IRQ_STATUS_EVENT_PENDING)
   {
      irqreturn = IRQ_HANDLED;
      dwInterruptFifoLevel = dwInterruptStatus & FBD_CXP_MASK_IRQ_STATUS_QUEUE_LEVEL;
      if (aslcxp_hw_debug) dev_printk(KERN_INFO, cxp_device->device, "%s: %u interrupts outstanding\n", __FUNCTION__, dwInterruptFifoLevel);
      while(dwInterruptFifoLevel-- > 0)
      {
         uint32_t irq_decoded;
         uint32_t dwInterrupt;

         irq_decoded = 0;
         dwInterrupt = aslcxp_read32(cxp_device, FBD_CXP_ADDR_IRQ_QUEUE);
         if (aslcxp_hw_debug) dev_printk(KERN_INFO, cxp_device->device, "%s: Interrupt queue entry = 0x%08X\n", __FUNCTION__, dwInterrupt);
         if (dwInterrupt & FBD_CXP_VALUE_IRQ_QUEUE_LINK_STATUS_CHANGE_EVENT)
         {
            dev_printk(KERN_INFO, cxp_device->device, "%s: link status change\n", __FUNCTION__);
            irq_decoded = 1;
         }
         if (dwInterrupt & FBD_CXP_VALUE_IRQ_QUEUE_ACK_EVENT)
         {
            if (aslcxp_hw_debug) dev_printk(KERN_INFO, cxp_device->device, "%s: ack interrupt\n", __FUNCTION__);
            wake_sleeper |= aslcxp_hw_process_device_reponse_locked(cxp_device);
            irq_decoded = 1;
         }
         if (dwInterrupt & FBD_CXP_VALUE_IRQ_QUEUE_GPIO_MESSAGE_EVENT)
         {
            uint32_t dwGpioValue = 0;
            uint32_t dwGpioStatus = 0;

            if (aslcxp_hw_debug) dev_printk(KERN_INFO, cxp_device->device, "%s: gpio interrupt\n", __FUNCTION__);
            dwGpioValue = dwInterrupt & FBD_CXP_MASK_IRQ_QUEUE_GPIO_MESSAGE_DATA;
            dwGpioStatus = aslcxp_read32(cxp_device, FBD_CXP_ADDR_HOST_DL_STATUS);
            irq_decoded = 1;
            /* Throw the GPIO event away CLR TODO CXPUART */
         }
         if (!irq_decoded){
            dev_err(cxp_device->device, "%s: Interrupt %08X not decoded\n", __FUNCTION__, dwInterrupt);
            aslcxp_hw_reset_locked(cxp_device);
            goto bail;
         }
      }
   }
bail:
   spin_unlock(&cxp_device->interrupt_spinlock);
   if (wake_sleeper) wake_up(&cxp_device->cxp_transaction.wait_queue_head);
   return irqreturn;
}

int aslcxp_complete_transaction_locked(struct cxp_transaction *transaction, long error_code, uint32_t protocol_error_code)
{
   int wake_sleeper = 0;

   switch(transaction->cxp_transaction_state){
      case cxp_idle:
      case cxp_complete:
      case cxp_error:
         break;
      default:
         transaction->cxp_transaction_state = error_code == 0 ? cxp_complete : cxp_error;
         transaction->error_code = error_code;
         transaction->protocol_error_code = protocol_error_code;
         wake_sleeper = 1;
         break;
   }
   return wake_sleeper;
}

void aslcxp_hw_restart_locked(struct aslcxp_device *cxp_device)
{
   aslcxp_hw_reset_locked(cxp_device);
   aslcxp_hw_start_locked(cxp_device);
   return;
}

void aslcxp_hw_panic_locked(struct aslcxp_device *cxp_device)
{
   aslcxp_write32(cxp_device, FBD_CXP_ADDR_HOST_DL_CONTROL, FBD_CXP_VALUE_HOST_CONTROL_GLOBAL_RESET);
   aslcxp_write32(cxp_device, FBD_CXP_ADDR_HOST_UL_CONTROL, FBD_CXP_VALUE_HOST_CONTROL_GLOBAL_RESET);
   udelay(2);

   aslcxp_write32(cxp_device, FBD_CXP_ADDR_HOST_DL_CONTROL, 0);
   aslcxp_write32(cxp_device, FBD_CXP_ADDR_HOST_UL_CONTROL, 0);
   udelay(2);

   cxp_device->shadow_interrupt_control = 0;

   return;
}

void aslcxp_hw_reset_locked(struct aslcxp_device *cxp_device)
{
   cxp_device->shadow_interrupt_control = 0;
   aslcxp_write32(cxp_device, FBD_CXP_ADDR_IRQ_CONTROL, cxp_device->shadow_interrupt_control);
   return;
}

void aslcxp_hw_start_locked(struct aslcxp_device *cxp_device)
{
   cxp_device->shadow_interrupt_control = FBD_CXP_VALUE_IRQ_CONTROL_LINK_STATUS_CHANGE_EN
                                        | FBD_CXP_VALUE_IRQ_CONTROL_ACK_EN
                                        | FBD_CXP_VALUE_IRQ_CONTROL_GPIO_EN
                                        | FBD_CXP_VALUE_IRQ_CONTROL_GLOBAL_EN;
   aslcxp_write32(cxp_device, FBD_CXP_ADDR_IRQ_CONTROL, cxp_device->shadow_interrupt_control);
   return;
}

