#define ASLENUM_VERSION_STRING "3.2.0.0"
#define ASLENUM_VERSION 0x03020000

#define ASLENUM_MAX_DEVICES 32

#define ASLENUM_STRING_FIREBIRD "FireBird"
#define ASLENUM_STRING_PHOENIX  "Phoenix"

#define ASLENUM_CONFIG_CACHE_SIZE 64

#define ASLENUM_WRITE_LOCK_TIMEOUT_DEFAULT 200 /* ms */

#define MDA_INVALID_READ		0xFFFFFFFF
#define MDA_INVALID_OFFSET		0xFFFFFFFF

#define MDA_MODULE_PREFIX "mda"
#define ASLCXP_DEVICE_NAME  "aslcxp"
#define ASLDMA_DEVICE_NAME  "asldma"
#define ASLENUM_DEVICE_NAME "aslenum"
#define ASLSER_DEVICE_NAME  "ttyCL"
#define ASLSER_DEVICE_TYPE_CAMERALINK 1
#define ASLSER_DRIVER_NAME  "aslser"
#define ASLSER_MAX_DEVICES   32
#define MDA_MAX_NAME_LENGTH 128


enum baseboard_type{
   board_type_unknown = 0,
   board_type_FireBird,
   board_type_Phoenix
};

struct mda_device_id{
   char           * driver_matching_name;
   kernel_ulong_t   device_name_type;
};

struct mda_device_params{
   u32                  memory_length;
   u32                  memory_offset;
   u32                  io_length;
   u32                  io_offset;
   u32                  interrupt_mask;
   u32                  interrupt_register;
};

struct mda_io_address{
   char *          kvaddr;
   size_t          kvsize;
};

struct mda_device{
   struct list_head  this_mda_device;
   char                     name[MDA_MAX_NAME_LENGTH];
   struct mda_device_id     id;
   struct mda_device_params params;
   struct device            device;
   struct mda_driver      * driver;
   struct mda_channel     * parent;
   struct mda_io_address    device_address;
   int                      channel;
   int                      instance;
};


struct mda_driver {
   struct list_head this_mda_driver;                                /* Internal use only */
   struct device_driver driver;                                                  /* Mandatory */
   const char *name;                                                        /* Mandatory */
   const struct mda_device_id *id_table;                                    /* Mandatory */
   int  (*probe)  (struct mda_device *dev, const struct mda_device_id *id); /* Mandatory */
   void (*remove) (struct mda_device *dev);                                 /* Mandatory */
   int  (*suspend) (struct mda_device *dev, pm_message_t state);                                /* Optional */
   int  (*suspend_late) (struct mda_device *dev, pm_message_t state);                           /* Optional */
   int  (*resume_early) (struct mda_device *dev);                           /* Optional */
   int  (*resume) (struct mda_device *dev);                                    /* Optional */
   void (*shutdown) (struct mda_device *dev);                               /* Optional */
   struct mda_error_handlers *error_handlers;                               /* Optional */
};
#define to_mda_driver(drv) container_of(drv, struct mda_driver, driver)
int mda_do_register_driver(struct mda_driver *mda_driver, struct module *module, const char *mod_name);
#define mda_register_driver(driver) mda_do_register_driver(driver, THIS_MODULE, KBUILD_MODNAME)
void mda_unregister_driver(struct mda_driver *mda_driver);

enum asldma_memory_type {
   system_memory,
   nvidia_p2p_memory
};

struct mda_dma_buffer{
   struct list_head        this_dma_buffer;
   u64                     user_addr_in;
   u64                     user_size_in;
   u64                     user_base;
   enum dma_data_direction direction;
   enum asldma_memory_type        memory_type;
   int                     nPages;
   int                     nSgElements;
   struct page *         * page_ptrs;
   struct scatterlist    * scatterlist;
   void                         (*destroy_callback)(void *data);
   void                         * destroy_data;
#if defined MDA_ENABLE_NVIDIA_P2P
   struct semaphore               destroy_sem;
   struct nvidia_p2p_page_table * nvidia_page_table;
   uint64_t                       nvidia_p2p_token;
   uint32_t                       nvidia_va_space;
   uint32_t                       nvidia_page_size;
   uint64_t                       nvidia_va_start;
   uint64_t                       nvidia_mapped_size;
#endif
};

struct aslenum_device {
   struct module *          owner;
   struct pci_dev *         pcidev;
   struct device *          device;
   enum baseboard_type      baseboard_type;
   struct mda_io_address    board_address;
   struct mda_io_address    ioport_address;
   struct mda_io_address    device_address;
   struct cdev              cdev;
   dev_t                    major;
   dev_t                    minor;

   int                      open_count;
   spinlock_t               open_lock;
   struct file *            file_write_exclusive;
   wait_queue_head_t        write_exclusive_wait_queue_head;

   spinlock_t               mda_root_node_lock;
   struct list_head         mda_root_node_list;
   unsigned char            config_space_cache[ASLENUM_CONFIG_CACHE_SIZE];
};

struct aslenum_file {
   struct aslenum_device *pEnumDevice;
   u32                   write_lock_timeout_ms;
   struct timer_list 	 write_lock_timer;
   enum {
      write_lock_idle,
      write_lock_waiting,
      write_lock_acquired,
      write_lock_timedout,
      write_lock_error
   }                     write_lock_state;
};

struct mda_root_node{
   struct list_head this_mda_root_node;
   char                    name[MDA_MAX_NAME_LENGTH];
   struct aslenum_device * parent;
   int                     instance;
   struct device           device;
   struct list_head channel_list;
   spinlock_t              lock;
   struct list_head        irq_register_list;
   spinlock_t              irq_lock;
   int                     fpga_is_reconfiguring;
};
#define to_mda_root_node(dev) container_of(dev, struct mda_root_node, device)

struct mda_channel{
   struct list_head       this_mda_channel;
   char                   name[MDA_MAX_NAME_LENGTH];
   struct mda_root_node * parent;
   int                    instance;
   int                    device_count;
   struct device          device;
   struct list_head       device_list;
   spinlock_t      lock;
};
#define to_mda_channel(dev) container_of(dev, struct mda_channel, device)

struct mda_device_info{
   kernel_ulong_t aslenum_driver_data;
   u16            pci_vendor_id;
   u16            pci_device_id;
   u16            pci_subvendor_id;
   u16            pci_subdevice_id;
   u8             pci_revision;
};

#define to_mda_device(dev) container_of(dev, struct mda_device, device)

struct mda_irq{
   struct list_head     this_mda_irq;
   struct mda_device  * mda_device;
   uint32_t             mask;
   irqreturn_t       (* mda_irq_handler)(struct mda_device *, void *);
   void               * data;
};

struct mda_irq_register{
   struct list_head        this_mda_irq_register;
   struct list_head        irq_function_list;
   char          *         kvaddr;
   uint32_t                offset;
   uint32_t                value;
};

long aslenum_ioctl_reglist(struct aslenum_device *baseboard, struct file * file, mda_io_buffer *user_io);
long aslenum_ioctl_get_child_name(struct aslenum_device *baseboard, mda_io_buffer *user_io);
long aslenum_ioctl_set_lock_timeout(struct aslenum_file *file, mda_io_buffer *user_io);

int aslenum_bus_init(void);
void aslenum_bus_remove(void);

int mda_create_nodes(struct aslenum_device *baseboard);
void mda_destroy_nodes(struct aslenum_device *baseboard);

int mda_baseboard_set_parameters(struct aslenum_device *baseboard, struct mda_device_params *params);
struct mda_root_node * mda_root_node_create(struct aslenum_device *baseboard, int index);
void mda_root_node_destroy(struct mda_root_node *mda_root_node);

int mda_parse_configuration(struct aslenum_device *baseboard, const char *configuration_date, ssize_t configuration_size);
void mda_destroy_channels(struct mda_root_node *root_node);
void mda_destroy_devices(struct mda_channel *channel);

struct mda_channel * mda_channel_create(struct mda_root_node *mda_root_node, int index);
void mda_channel_destroy(struct mda_channel *mda_root_node);

struct mda_device * mda_device_create(struct mda_channel *channel, struct mda_device_id *id, struct mda_device_params *params);
int aslenum_check_exclusive(struct aslenum_device *baseboard, struct file * file);

long mda_ioctl_reglist(struct mda_device *device, mda_io_buffer *user_io, int writeok);
long mda_ioctl_version(mda_io_buffer *user_io, ui32 dwVersion);
long mda_ioctl_unique_id(mda_io_buffer *user_io, struct mda_device *pMdaDevice);

irqreturn_t aslenum_isr(int irq, void *data);
irqreturn_t aslenum_firebird_isr(struct aslenum_device *firebird);

typedef struct aslenum_device tAslEnumDevice;
typedef struct mda_root_node  tMdaRootNode;
typedef struct mda_channel    tMdaChannel;

#define MDA_CONFIG_OK     0
#define MDA_CONFIG_ERROR -EIO

