#include "precomp.h"

static __inline irqreturn_t firebird_update_irqreturn(irqreturn_t old, irqreturn_t latest)
{
   irqreturn_t updated = old;

   if (latest == IRQ_WAKE_THREAD) updated = latest;
   else if ((latest == IRQ_HANDLED) && (old == IRQ_NONE)) updated = latest;
   return updated;
}

irqreturn_t aslenum_firebird_node_isr(struct mda_root_node *root_node)
{
   irqreturn_t root_node_irqreturn = IRQ_NONE;
   struct mda_irq_register *irq_register = NULL;

   spin_lock(&root_node->irq_lock);
   list_for_each_entry(irq_register, &root_node->irq_register_list, this_mda_irq_register){
      if (irq_register->kvaddr) irq_register->value = __raw_readl(irq_register->kvaddr);
      else irq_register->value = MDA_INVALID_READ;

      if (irq_register->value == MDA_INVALID_READ) irq_register->value = 0;

      if (irq_register->value){
         struct mda_irq *irq = NULL;
         list_for_each_entry(irq, &irq_register->irq_function_list, this_mda_irq){
            if (irq_register->value & irq->mask){
               irqreturn_t irqreturn = irq->mda_irq_handler(irq->mda_device, irq->data);
               root_node_irqreturn = firebird_update_irqreturn(root_node_irqreturn, irqreturn);
            }
         }
      }
   }
   spin_unlock(&root_node->irq_lock);
   return root_node_irqreturn;
}

irqreturn_t aslenum_firebird_isr(struct aslenum_device *firebird)
{
   irqreturn_t firebird_irqreturn = IRQ_NONE;
   struct mda_root_node *root_node = NULL;

   list_for_each_entry(root_node, &firebird->mda_root_node_list, this_mda_root_node){
      irqreturn_t irqreturn = aslenum_firebird_node_isr(root_node);
      firebird_irqreturn = firebird_update_irqreturn(firebird_irqreturn, irqreturn);
   }
   return firebird_irqreturn;
}
