#include "precomp.h"

static __inline irqreturn_t phoenix_update_irqreturn(irqreturn_t old, irqreturn_t latest)
{
   irqreturn_t updated = old;

   if (latest == IRQ_WAKE_THREAD) updated = latest;
   else if ((latest == IRQ_HANDLED) && (old == IRQ_NONE)) updated = latest;
   return updated;
}

static __inline irqreturn_t aslenum_phoenix_node_isr(struct mda_root_node *root_node)
{
   irqreturn_t root_node_irqreturn = IRQ_NONE;
   struct mda_irq_register *irq_register = NULL;

   spin_lock(&root_node->irq_lock);

   if (root_node->fpga_is_reconfiguring) goto bail;

   list_for_each_entry(irq_register, &root_node->irq_register_list, this_mda_irq_register){
      struct mda_irq *irq = NULL;
      list_for_each_entry(irq, &irq_register->irq_function_list, this_mda_irq){
         irqreturn_t irqreturn = irq->mda_irq_handler(irq->mda_device, irq->data);
         root_node_irqreturn = phoenix_update_irqreturn(root_node_irqreturn, irqreturn);
      }
   }

bail:
   spin_unlock(&root_node->irq_lock);
   return root_node_irqreturn;
}

irqreturn_t aslenum_phoenix_isr(struct aslenum_device *phoenix)
{
   irqreturn_t phoenix_irqreturn = IRQ_NONE;
   struct mda_root_node *root_node = NULL;

   list_for_each_entry(root_node, &phoenix->mda_root_node_list, this_mda_root_node){
      irqreturn_t irqreturn = aslenum_phoenix_node_isr(root_node);
      phoenix_irqreturn = phoenix_update_irqreturn(phoenix_irqreturn, irqreturn);
   }
   return phoenix_irqreturn;
}

void mda_fpga_is_reconfiguring(struct mda_device *mda_device, int is_reconfiguring)
{
   unsigned long saved_irq_flags = 0;
   struct mda_root_node *root_node = mda_device->parent->parent;

   spin_lock_irqsave(&root_node->irq_lock, saved_irq_flags);
   root_node->fpga_is_reconfiguring = is_reconfiguring;
   spin_unlock_irqrestore(&root_node->irq_lock, saved_irq_flags);

   return;
}

int mda_config_space_restore(struct mda_device *mda_device)
{
   int error = -EIO;
   struct aslenum_device *baseboard = mda_device->parent->parent->parent;
   uint32_t index = 0;
   uint32_t dword_total = sizeof(baseboard->config_space_cache) / sizeof(uint32_t);

   while(index < dword_total){
      uint32_t config_offset = index * sizeof(uint32_t);
      uint32_t *cache_address = (uint32_t*)&(baseboard->config_space_cache[config_offset]);
      error = pci_write_config_dword(baseboard->pcidev, config_offset, *cache_address);
      if (error){
         dev_err(&mda_device->device, "%s: pci_write_config_dword failed with %d\n", __FUNCTION__, error);
         goto bail;
      }
   }
bail:
   return error;
}


EXPORT_SYMBOL(aslenum_phoenix_isr);
EXPORT_SYMBOL(mda_fpga_is_reconfiguring);
EXPORT_SYMBOL(mda_config_space_restore);

