/****************************************************************************
 *
 * ACTIVE SILICON LIMITED
 *
 * File name   : mda_hw.h
 * Function    : Hardware register structure and definitions.
 * Project     : AP20
 * Authors     : Martin Bone
 * Systems     : ANSI C
 *
 * Copyright (c) 2000 - 2012 Active Silicon Ltd.
 ****************************************************************************
 * Comments:
 * --------
 *
 ****************************************************************************
 */


#ifndef _MDA_HW_H_
#define _MDA_HW_H_

typedef enum {
   MDA_ADDR_PCIE_VENDOR_ID       = 0x0000,
   MDA_ADDR_PCIE_DEVICE_ID       = 0x0002,
   MDA_ADDR_PCIE_REVISION_ID     = 0x0008,
   MDA_ADDR_PCIE_LINK_CAP        = 0x000C,
   MDA_ADDR_PCIE_LINK_STAT       = 0x0012,
   MDA_ADDR_PCIE_SUB_VENDOR_ID   = 0x002C,
   MDA_ADDR_PCIE_SUB_DEVICE_ID   = 0x002E,
   MDA_ADDR_PCIE_BASE_CAP_PTR    = 0x0034,   /* "Standard" PCI Capabilities  */
   MDA_ADDR_PCIE_BASE_EXTCAP_PTR = 0x0100    /* "Extended" PCIe Capabilities */
} MDA_PcieAddrMap;


/* Standard Capabilities */
#define _MDA_GetCapPtr(_x)    (((_x) & 0x0000FF00 ) >> 8 )
#define _MDA_GetCapId(_x)      ((_x) & 0x000000FF )

/* Extended Capabilities */
#define _MDA_GetExtCapPtr(_x) (((_x) & 0xFFF00000 ) >> 20 )
#define _MDA_GetExtCapRev(_x) (((_x) & 0x000F0000 ) >> 16 )
#define _MDA_GetExtCapId(_x)   ((_x) & 0x0000FFFF )

#define _MDA_VsecIdAddr(_x)    ((_x) + 4)
#define _MDA_VsecDataAddr(_x)  ((_x) + 8)


typedef enum {
   MDA_HMASK_PCIE_SPEED          = 0x000F,
   MDA_HBIT_PCIE_SPEED_GEN1      = 0x0001,
   MDA_HBIT_PCIE_SPEED_GEN2      = 0x0002,
   MDA_HBIT_PCIE_SPEED_GEN3      = 0x0003,

   MDA_HMASK_PCIE_WIDTH          = 0x02F0,
   MDA_HBIT_PCIE_X1              = 0x0010,
   MDA_HBIT_PCIE_X2              = 0x0020,
   MDA_HBIT_PCIE_X4              = 0x0040,
   MDA_HBIT_PCIE_X8              = 0x0080,
   MDA_HBIT_PCIE_X12             = 0x00C0,
   MDA_HBIT_PCIE_X16             = 0x0100,
   MDA_HBIT_PCIE_X32             = 0x0200
} MDA_PcieBitMap;


/* This is a "complete" list of all the PCIe capability IDs.
 * Whilst very few are used in the code the full list is retained for reference
 */
typedef enum {
   MDA_PCIE_CAP_ID_PM      = 0x01,     /* Power Management                    */
   MDA_PCIE_CAP_ID_AGP     = 0x02,     /* Accelerated Graphics Port           */
   MDA_PCIE_CAP_ID_VPD     = 0x03,     /* Vital Product Data                  */
   MDA_PCIE_CAP_ID_SLOTID  = 0x04,     /* Slot Identification                 */
   MDA_PCIE_CAP_ID_MSI     = 0x05,     /* Message Signalled Interrupts        */
   MDA_PCIE_CAP_ID_CHSWP   = 0x06,     /* CompactPCI HotSwap                  */
   MDA_PCIE_CAP_ID_PCIX    = 0x07,     /* PCI-X                               */
   MDA_PCIE_CAP_ID_HT      = 0x08,     /* HyperTransport                      */
   MDA_PCIE_CAP_ID_VNDR    = 0x09,     /* Vendor specific                     */
   MDA_PCIE_CAP_ID_DBG     = 0x0A,     /* Debug port                          */
   MDA_PCIE_CAP_ID_CCRC    = 0x0B,     /* CompactPCI Central Resource Control */
   MDA_PCIE_CAP_ID_SHPC    = 0x0C,     /* PCI Standard Hot-Plug Controller    */
   MDA_PCIE_CAP_ID_SSVID   = 0x0D,     /* Bridge subsystem vendor/device ID   */
   MDA_PCIE_CAP_ID_AGP3    = 0x0E,     /* AGP Target PCI-PCI bridge           */
   MDA_PCIE_CAP_ID_EXP     = 0x10,     /* PCI Express                         */
   MDA_PCIE_CAP_ID_MSIX    = 0x11,     /* MSI-X                               */
   MDA_PCIE_CAP_ID_AF      = 0x13      /* PCI Advanced Features               */
} MDA_PcieCapsIDs;


typedef enum {
   MDA_PCIE_EXTCAP_VSEC    = 0x000B
} MDA_PcieExtCapIDs;

typedef enum {
   MDA_PCIE_VSEC_SPI_FLASH = 0x8000
} MDA_PcieVsecIDs;

#endif /* _MDA_HW_H_ */
