#ifndef ASL_DMA_IOCTL_H
#define ASL_DMA_IOCTL_H

#include "mda_linux.h"

#define _MDADMA_IOCTL(_n)     _IOWR(MDADMA_TYPE, (MDADMA_FCMIN + (_n)), mda_io_buffer)


/* output_buffer -> struct MDA_SIoctlDeviceInfo
 */
#define CDA_IOCTL_DEVICEINFO              _MDADMA_IOCTL( 0x00 )


/* input_buffer  -> register list
 * output_buffer -> register list
 */
#define CDA_IOCTL_REGLIST                 _MDADMA_IOCTL( 0x01 )


/* Restore PCI config registers to state when driver loaded
 */
#define MDA_IOCTL_CONFIG_RESTORE          _MDADMA_IOCTL( 0x02 )


/* Claim a device exclusively
 */
#define CDA_IOCTL_CLAIM                   _MDADMA_IOCTL( 0x03 )


/* Release a previously claimed device
 */
#define MDA_IOCTL_RELEASE                 _MDADMA_IOCTL( 0x04 )


/* param_buffer  -> SG Table Header
 * output_buffer -> SG address length pairs
 */
#define MDA_IOCTL_MEMORY_LOCK_RISC        _MDADMA_IOCTL( 0x05 )


/* param_buffer  -> SG Table Header
 * output_buffer -> SG address length pairs
 */
#define MDA_IOCTL_MEMORY_LOCK_IMAGE       _MDADMA_IOCTL( 0x06 )


/* param_buffer  -> SG Table Header
 */
#define MDA_IOCTL_MEMORY_FLUSH            _MDADMA_IOCTL( 0x07 )


/* param_buffer   -> size_t
 * output_buffer  -> void *
 */
#define MDA_IOCTL_MALLOC32                _MDADMA_IOCTL( 0x08 )


/* param_buffer   -> void *
 */
#define MDA_IOCTL_FREE32                  _MDADMA_IOCTL( 0x09 )


/* param_buffer   -> size_t
 * output_buffer  -> void *
 */
#define MDA_IOCTL_MALLOC64                _MDADMA_IOCTL( 0x0A )


/* param_buffer   -> void *
 */
#define MDA_IOCTL_FREE64                  _MDADMA_IOCTL( 0x0B )


/* output_buffer -> MDA_SIoctlEvent
 */
#define MDA_IOCTL_MDA_EVENT               _MDADMA_IOCTL( 0x0C )


/* param_buffer -> MDA_SIoctlEvent
 */
#define CDA_IOCTL_EVENT_PUT               _MDADMA_IOCTL( 0x0D )


/* putput_buffer -> MDA_SIoctlPowerEvent
 */
#define MDA_IOCTL_POWER_EVENT             _MDADMA_IOCTL( 0x0E )


/* param_buffer -> MDA_SIoctlPowerEvent
 */
#define MDA_IOCTL_POWER_ACKNOWLEDGE       _MDADMA_IOCTL( 0x0F )


/* output_buffer -> MDA_SDMAInfo */
#define MDA_IOCTL_DMA_INFO                _MDADMA_IOCTL( 0x10 )


#define MDA_IOCTL_CLEAR_EVENT_QUEUE       _MDADMA_IOCTL( 0x11 )


/* param_buffer  -> SG Table Header
 */
#define MDA_IOCTL_MEMORY_UNLOCK           _MDADMA_IOCTL( 0x12 )


#define MDA_IOCTL_EVENT_EXIT              _MDADMA_IOCTL( 0x13 )


/* Enable Disable DMA interrupt event processing for FPGA reconfiguration
 */
#define MDA_IOCTL_EVENT_ENABLE            _MDADMA_IOCTL( 0x14 )


#define MDA_IOCTL_EVENT_DISABLE           _MDADMA_IOCTL( 0x15 )

#define MDA_IOCTL_GET_TIME                _MDADMA_IOCTL( 0x16 )
/* output buffer -> uint64_t */


/* param_buffer  -> SG Table Header
 * output_buffer -> SG address length pairs
 */
#define MDA_IOCTL_MEMORY_LOCK_CUDA        _MDADMA_IOCTL( 0x17 )


#endif /* ASL_DMA_IOCTL_H */

